"""
Copyright (c) 2014 Maciej Nabozny

This file is part of CloudOver project.

CloudOver is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


from coretalk.models.coretalk.userdata import UserData

from corecluster.models.core.vm import VM
from corecluster.utils.decorators import register
from corecluster.utils import validation as v


@register(auth='token', validate={'name': v.is_string()})
def create(context, name, data):
    userdata = UserData()
    userdata.name = name
    userdata.data = data
    userdata.user_id = context.user_id
    userdata.save()

    return userdata.to_dict


@register(auth='token', validate={'userdata_id': v.is_id()})
def edit(context, userdata_id, **kwargs):
    """ Edit VM properties """
    ud = UserData.get(context.user_id, userdata_id)
    ud.edit(context, **kwargs)
    ud.save()

    return ud.to_dict


@register(auth='token', validate={'userdata_id': v.is_id()})
def delete(context, userdata_id):
    userdata = UserData.get(context.user_id, userdata_id)
    userdata.remove(context)


@register(auth="token")
def get_list(context):
    """
    Returns list of all userdata objects in cloud
    """
    userdata = UserData.get_list(context.user_id)
    return [ud.to_dict for ud in userdata]


@register(auth="token", validate={'userdata_id': v.is_id()})
def get_by_id(context, userdata_id):
    """
    Returns userdata object identified by id
    """
    return UserData.get(context.user_id, userdata_id).to_dict


@register(auth='token', validate={'userdata_id': v.is_id(), 'vm_id': v.is_id()})
def attach(context, userdata_id, vm_id):
    """
    Attach userdata object to vm (it is accessible via http://169.254.169.254/latest/user-data/)
    :param userdata_id: id of userdata object
    :param vm_id: id ov vm
    """
    userdata = UserData.get(context.user_id, userdata_id)
    vm = VM.get(context.user_id, vm_id)

    vm.set_prop('coretalk_user_data', userdata_id)
    vm.save()


@register(auth='token', validate={'vm_id': v.is_id()})
def detach(context, vm_id):
    """ Detach userdata from vm instance
    :param vm_id: id of virtual machine
    :return:
    """
    vm = VM.get(context.user_id, vm_id)

    all_props = vm.get_all_props()
    if 'coretalk_user_data' in all_props:
        del all_props['coretalk_user_data']

    vm.set_all_props(all_props)
    vm.save()


@register(auth='token')
def describe(context):
    """ Show serializable and editable fields in model """
    return UserData.describe_model()